@@ -72,7 +72,7 @@ gint sessions = 0;		/* Number of remote sessions */
 gint flexi_servers = 0;		/* Number of flexi servers */
 uid_t GdmUserId;		/* Userid under which gdm should run */
 gid_t GdmGroupId;		/* Groupid under which gdm should run */
-pid_t extra_process = -1;	/* An extra process.  Used for quickie 
+pid_t extra_process = 0;	/* An extra process.  Used for quickie 
 				   processes, so that they also get whacked */
 int extra_status = 0;		/* Last status from the last extra process */
 pid_t gdm_main_pid = 0;
@@ -82,6 +82,8 @@ gboolean no_daemon = FALSE;	/* Do not daemonize */
 GdmConnection *fifoconn = NULL; /* Fifo connection */
 GdmConnection *unixconn = NULL; /* UNIX Socket connection */
 
+char *gdm_charset = NULL;
+
 /* True if the server that was run was in actuallity not specified in the
  * config file.  That is if xdmcp was disabled and no local servers were
  * defined.  If the user kills all his local servers by mistake and keeps
@@ -133,7 +135,6 @@ gchar *GdmUserAuthFB = NULL;
 gchar *GdmPidFile = NULL;
 gchar *GdmDefaultPath = NULL;
 gchar *GdmRootPath = NULL;
-gchar *GdmDefaultLocale = NULL;
 gboolean  GdmKillInitClients = FALSE;
 gint  GdmUserMaxFile = 0;
 gint  GdmSessionMaxFile = 0;
@@ -160,6 +161,9 @@ gint GdmTimedLoginDelay = 0;
 gchar *GdmStandardXServer = NULL;
 gint  GdmFlexibleXServers = 5;
 gchar *GdmXnest = NULL;
+int GdmFirstVT = 7;
+gboolean GdmVTAllocation = TRUE;
+
 
 /* set in the main function */
 char **stored_argv = NULL;
@@ -255,7 +259,6 @@ gdm_config_parse (void)
     GdmReboot = gnome_config_get_string (GDM_KEY_REBOOT);
     GdmRetryDelay = gnome_config_get_int (GDM_KEY_RETRYDELAY);
     GdmRootPath = gnome_config_get_string (GDM_KEY_ROOTPATH);
-    GdmDefaultLocale = gnome_config_get_string (GDM_KEY_LOCALE);
     GdmServAuthDir = gnome_config_get_string (GDM_KEY_SERVAUTH);
     GdmSessDir = gnome_config_get_string (GDM_KEY_SESSDIR);
     GdmSuspend = gnome_config_get_string (GDM_KEY_SUSPEND);
@@ -310,6 +313,9 @@ gdm_config_parse (void)
     if (ve_string_empty (GdmXnest))
 	    GdmXnest = NULL;
 
+    GdmFirstVT = gnome_config_get_int (GDM_KEY_FIRSTVT);    
+    GdmVTAllocation = gnome_config_get_bool (GDM_KEY_VTALLOCATION);    
+
     GdmDebug = gnome_config_get_bool (GDM_KEY_DEBUG);
 
     gnome_config_pop_prefix();
@@ -789,21 +795,20 @@ void
 gdm_final_cleanup (void)
 {
 	GSList *list, *li;
-	sigset_t mask;
 
 	gdm_debug ("gdm_final_cleanup");
 
-	sigemptyset (&mask);
-	sigaddset (&mask, SIGCHLD);
-	sigprocmask (SIG_BLOCK, &mask, NULL); 
+	gdm_sigchld_block_push ();
 
 	if (extra_process > 1) {
 		/* we sigterm extra processes, and we
 		 * don't wait */
-		kill (extra_process, SIGTERM);
-		extra_process = -1;
+		kill (-(extra_process), SIGTERM);
+		extra_process = 0;
 	}
 
+	gdm_sigchld_block_pop ();
+
 	list = g_slist_copy (displays);
 	for (li = list; li != NULL; li = li->next) {
 		GdmDisplay *d = li->data;
@@ -1001,7 +1006,7 @@ gdm_cleanup_children (void)
 
     if (pid == extra_process) {
 	    /* an extra process died, yay! */
-	    extra_process = -1;
+	    extra_process = 0;
 	    extra_status = exitstatus;
 	    return TRUE;
     }
@@ -1016,26 +1021,22 @@ gdm_cleanup_children (void)
 	    gdm_debug ("gdm_cleanup_children: Slave crashed, killing it's "
 		       "children");
 
-	    if (d->sesspid > 0) {
-		    if (kill (d->sesspid, SIGTERM) == 0)
+	    if (d->sesspid > 1 &&
+		kill (d->sesspid, SIGTERM) == 0)
 			    ve_waitpid_no_signal (d->sesspid, NULL, 0);
-		    d->sesspid = 0;
-	    }
-	    if (d->greetpid > 0) {
-		    if (kill (d->greetpid, SIGTERM) == 0)
+	    d->sesspid = 0;
+	    if (d->greetpid > 1 &&
+		kill (d->greetpid, SIGTERM) == 0)
 			    ve_waitpid_no_signal (d->greetpid, NULL, 0);
-		    d->greetpid = 0;
-	    }
-	    if (d->chooserpid > 0) {
-		    if (kill (d->chooserpid, SIGTERM) == 0)
+	    d->greetpid = 0;
+	    if (d->chooserpid > 1 &&
+		kill (d->chooserpid, SIGTERM) == 0)
 			    ve_waitpid_no_signal (d->chooserpid, NULL, 0);
-		    d->chooserpid = 0;
-	    }
-	    if (d->servpid > 0) {
-		    if (kill (d->servpid, SIGTERM) == 0)
+	    d->chooserpid = 0;
+	    if (d->servpid > 1 &&
+		kill (d->servpid, SIGTERM) == 0)
 			    ve_waitpid_no_signal (d->servpid, NULL, 0);
-		    d->servpid = 0;
-	    }
+	    d->servpid = 0;
     }
 
     /* null all these, they are not valid most definately */
@@ -1408,6 +1409,7 @@ main (int argc, char *argv[])
     FILE *pf;
     poptContext ctx;
     int nextopt;
+    const char *charset;
 
     gdm_main_pid = getpid ();
 
@@ -1415,6 +1417,7 @@ main (int argc, char *argv[])
     ensure_desc_012 ();
 
     store_argv (argc, argv);
+    gdm_saveenv ();
 
     bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
     textdomain (GETTEXT_PACKAGE);
@@ -1448,6 +1451,9 @@ main (int argc, char *argv[])
     main_loop = g_main_loop_new (NULL, FALSE);
     openlog ("gdm", LOG_PID, LOG_DAEMON);
 
+    if ( ! g_get_charset (&charset)) {
+	    gdm_charset = g_strdup (charset);
+    }
 
     /* Parse configuration file */
     gdm_config_parse();
@@ -1462,7 +1468,7 @@ main (int argc, char *argv[])
 
         if (pf != NULL &&
 	    fscanf (pf, "%d", &pidv) == 1 &&
-	    kill (pidv, 0) != -1 &&
+	    kill (pidv, 0) == 0 &&
 	    linux_only_is_running (pidv)) {
 		/* make sure the pid file doesn't get wiped */
 		GdmPidFile = NULL;
@@ -1482,6 +1488,9 @@ main (int argc, char *argv[])
 	    fprintf (pf, "%d\n", (int)getpid());
 	    fclose (pf);
 	}
+
+	chdir (GdmServAuthDir);
+	umask (022);
     }
     else
 	gdm_daemonify();
@@ -1620,7 +1629,8 @@ write_x_servers (GdmDisplay *d)
 		char **argv;
 		char *command;
 		argv = gdm_server_resolve_command_line
-			(d, FALSE /* resolve_handled */);
+			(d, FALSE /* resolve_handled */,
+			 NULL /* vtarg */);
 		command = g_strjoinv (" ", argv);
 		g_strfreev (argv);
 		fprintf (fp, "%s local %s\n", d->name, command);
@@ -1639,9 +1649,11 @@ send_slave_ack (GdmDisplay *d)
 		not[1] = '\n';
 		write (d->master_notify_fd, not, 2);
 	}
-	if (d->slavepid > 0) {
+	gdm_sigchld_block_push ();
+	if (d->slavepid > 1) {
 		kill (d->slavepid, SIGUSR2);
 	}
+	gdm_sigchld_block_pop ();
 }
 
 static void
@@ -1654,9 +1666,11 @@ send_slave_command (GdmDisplay *d, const char *command)
 		write (d->master_notify_fd, cmd, strlen (cmd));
 		g_free (cmd);
 	}
-	if (d->slavepid > 0) {
+	gdm_sigchld_block_push ();
+	if (d->slavepid > 1) {
 		kill (d->slavepid, SIGUSR2);
 	}
+	gdm_sigchld_block_pop ();
 }
 
 
@@ -1817,9 +1831,7 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 		d = gdm_display_lookup (slave_pid);
 
 		if (d != NULL) {
-#ifdef __linux__
 			d->vt = vt_num;
-#endif
 			gdm_debug ("Got VT_NUM == %d", vt_num);
 			/* send ack */
 			send_slave_ack (d);
@@ -1991,8 +2003,10 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 		GSList *li;
 		for (li = displays; li != NULL; li = li->next) {
 			GdmDisplay *d = li->data;
-			if (d->greetpid > 0)
+			gdm_sigchld_block_push ();
+			if (d->greetpid > 1)
 				kill (d->greetpid, SIGHUP);
+			gdm_sigchld_block_pop ();
 		}
 	} else if (strncmp (msg, GDM_SOP_WRITE_X_SERVERS " ",
 		            strlen (GDM_SOP_WRITE_X_SERVERS " ")) == 0) {
@@ -2370,8 +2384,10 @@ notify_displays_int (const char *key, int val)
 			gdm_fdprintf (disp->master_notify_fd,
 				      "%c%s %d\n",
 				      GDM_SLAVE_NOTIFY_KEY, key, val);
-			if (disp->slavepid > 0)
+			gdm_sigchld_block_push ();
+			if (disp->slavepid > 1)
 				kill (disp->slavepid, SIGUSR2);
+			gdm_sigchld_block_pop ();
 		}
 	}
 }
@@ -2386,8 +2402,10 @@ notify_displays_string (const char *key, const char *val)
 			gdm_fdprintf (disp->master_notify_fd,
 				      "%c%s %s\n",
 				      GDM_SLAVE_NOTIFY_KEY, key, val);
-			if (disp->slavepid > 0)
+			gdm_sigchld_block_push ();
+			if (disp->slavepid > 1)
 				kill (disp->slavepid, SIGUSR2);
+			gdm_sigchld_block_pop ();
 		}
 	}
 }
@@ -2716,12 +2734,7 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 					 ve_sure_string (disp->xnest_disp));
 			} else {
 				gdm_connection_printf (conn, "%d",
-#ifdef __linux__
-						       disp->vt
-#else
-						       -1
-#endif
-						      );
+						       disp->vt);
 			}
 		}
 		gdm_connection_write (conn, "\n");
