@@ -26,6 +26,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <fcntl.h>
 #include <signal.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
@@ -39,6 +40,17 @@
 #include "gdm.h"
 #include "gdmcomm.h"
 
+static pid_t xnest_pid = 0;
+
+static void
+term_handler (int sig)
+{
+	if (xnest_pid != 0)
+		kill (xnest_pid, SIGTERM);
+	else
+		exit (0);
+}
+
 static int
 get_free_display (void)
 {
@@ -156,6 +168,39 @@ test_opt (const char *cmd, const char *help, const char *option)
 }
 #endif
 
+static char *
+get_font_path (const char *display)
+{
+	Display *disp;
+	char **font_path;
+	int n_fonts;
+	int i;
+	GString *gs;
+
+	disp = XOpenDisplay (display);
+	if (disp == NULL)
+		return NULL;
+
+	font_path = XGetFontPath (disp, &n_fonts);
+	if (font_path == NULL) {
+		XCloseDisplay (disp);
+		return NULL;
+	}
+
+	gs = g_string_new (NULL);
+	for (i = 0; i < n_fonts; i++) {
+		if (i != 0)
+			g_string_append_c (gs, ',');
+		g_string_append (gs, font_path[i]);
+	}
+
+	XFreeFontPath (font_path);
+
+	XCloseDisplay (disp);
+
+	return g_string_free (gs, FALSE);
+}
+
 static char **
 make_us_an_exec_vector (const char *xnest)
 {
@@ -164,6 +209,8 @@ make_us_an_exec_vector (const char *xnest)
 	int argc;
 	char **xnest_vec;
 	char **options_vec = NULL;
+	gboolean got_font_path = FALSE;
+	char *font_path = NULL;
 
 	if( ! ve_string_empty (xnest_binary))
 		xnest = xnest_binary;
@@ -180,7 +227,8 @@ make_us_an_exec_vector (const char *xnest)
 	argc = ve_vector_len (xnest_vec) +
 		1 +
 		ve_vector_len (options_vec) +
-		3;
+		3 +
+		2;
 
 	vector = g_new0 (char *, argc);
 
@@ -193,11 +241,15 @@ make_us_an_exec_vector (const char *xnest)
 
 	for (i = 1; xnest_vec[i] != NULL; i++) {
 		vector[ii++] = xnest_vec[i];
+		if (strcmp (xnest_vec[i], "-fp") == 0)
+			got_font_path = TRUE;
 	}
 
 	if (options_vec != NULL) {
 		for (i = 0; options_vec[i] != NULL; i++) {
 			vector[ii++] = options_vec[i];
+			if (strcmp (options_vec[i], "-fp") == 0)
+				got_font_path = TRUE;
 		}
 	}
 
@@ -213,6 +265,14 @@ make_us_an_exec_vector (const char *xnest)
 		}
 	}
 
+	if ( ! got_font_path)
+		font_path = get_font_path (NULL);
+
+	if (font_path != NULL) {
+		vector[ii++] = "-fp";
+		vector[ii++] = font_path;
+	}
+
 	return vector;
 }
 
@@ -355,7 +415,7 @@ main (int argc, char *argv[])
 	const char **args;
 	char *xnest;
 	char **execvec;
-	pid_t pid;
+	struct sigaction term;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -485,6 +545,7 @@ main (int argc, char *argv[])
 	}
 
 	printf ("DISPLAY=:%d\n", display);
+	fflush (stdout);
 
 	g_snprintf (display_num, sizeof (display_num), ":%d", display);
 	g_snprintf (indirect_host, sizeof (indirect_host), "%s", host);
@@ -498,10 +559,24 @@ main (int argc, char *argv[])
 		if (fork () > 0) {
 			_exit (0);
 		}
+		close (0);
+		close (1);
+		close (2);
+		open ("/dev/null", O_RDWR);
+		open ("/dev/null", O_RDONLY);
+		open ("/dev/null", O_RDONLY);
 	}
 
-	pid = fork ();
-	if (pid == 0) {
+	term.sa_handler = term_handler;
+	term.sa_flags = SA_RESTART;
+	sigemptyset (&term.sa_mask);
+
+	sigaction (SIGTERM, &term, NULL);
+	sigaction (SIGINT, &term, NULL);
+	sigaction (SIGHUP, &term, NULL);
+
+	xnest_pid = fork ();
+	if (xnest_pid == 0) {
 		execvp (execvec[0], execvec);
 		g_warning ("Can't exec, trying Xnest");
 		execvec[0] = "Xnest";
@@ -509,13 +584,14 @@ main (int argc, char *argv[])
 		g_warning ("Can't exec that either, giving up");
 		/* FIXME: this should be handled in the GUI */
 		_exit (1);
-	} else if (pid < 0) {
+	} else if (xnest_pid < 0) {
 		/* eeeek */
 		g_warning ("Can't fork");
 		_exit (1);
 	}
 
-	ve_waitpid_no_signal (pid, 0, 0);
+	ve_waitpid_no_signal (xnest_pid, 0, 0);
+	xnest_pid = 0;
 
 	socket = g_strdup_printf ("/tmp/.X11-unix/X%d", display);
 	unlink (socket);
